/*
 * arch/arch/mach-ixp4xx/ng-pci.c
 *
 * PCI setup routines for Netgear FVX538/FVS338
 *
 * Copyright (c) 2006, TeamF1, Inc.
 * Copyright (C) 2002 Jungo Software Technologies.
 * Copyright (C) 2003 MontaVista Softwrae, Inc.
 *
 * Maintainer: Deepak Saxena <dsaxena@mvista.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#include <linux/pci.h>
#include <linux/init.h>

#include <asm/mach-types.h>
#include <asm/hardware.h>
#include <asm/irq.h>

#include <asm/mach/pci.h>

extern void ixp4xx_pci_preinit(void);
extern int ixp4xx_setup(int nr, struct pci_sys_data *sys);
extern struct pci_bus *ixp4xx_scan_bus(int nr, struct pci_sys_data *sys);

void __init ng_pci_preinit(void)
{
    gpio_line_config(NG_PCI_SLOT0_PIN,
                     IXP4XX_GPIO_IN | IXP4XX_GPIO_ACTIVE_LOW);

    gpio_line_config(NG_PCI_SLOT1_PIN,
                     IXP4XX_GPIO_IN | IXP4XX_GPIO_ACTIVE_LOW);

    gpio_line_isr_clear(NG_PCI_SLOT0_PIN);
    gpio_line_isr_clear(NG_PCI_SLOT1_PIN);

    ixp4xx_pci_preinit();
}

static int __init ng_map_irq(struct pci_dev *dev, u8 slot, u8 pin)
{
    if (slot == NG_PCI_SLOT0_DEVID)
        return IRQ_NG_PCI_SLOT0;
    else if (slot == NG_PCI_SLOT1_DEVID)
        return IRQ_NG_PCI_SLOT1;
    return -1;
}

struct hw_pci ng_pci __initdata = {
    .nr_controllers = 1,
    .preinit =        ng_pci_preinit,
    .swizzle =        pci_std_swizzle,
    .setup =          ixp4xx_setup,
    .scan =           ixp4xx_scan_bus,
    .map_irq =        ng_map_irq,
};

int __init ng_pci_init(void)
{
    if (machine_is_ng_fvx538())
        pci_common_init(&ng_pci);
    return 0;
}

subsys_initcall(ng_pci_init);
